@@ -1,7 +1,7 @@
 /*
- *  Linux/m68k Frame Buffer Device Configuration
+ *  Linux Frame Buffer Device Configuration
  *
- *   Copyright 1995 by Geert Uytterhoeven
+ *   Copyright 1995-1998 by Geert Uytterhoeven
  *		       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
  *
  *  --------------------------------------------------------------------------
@@ -17,12 +17,13 @@
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <sys/ioctl.h>
 
 struct file;
 struct inode;
 
-#include <linux/fb.h>
+#include "fb.h"
 
 #include "fbset.h"
 
@@ -31,7 +32,7 @@ struct inode;
      *  Default Frame Buffer Special Device Node
      */
 
-#define DEFAULT_FRAMEBUFFER	"/dev/fb0current"
+#define DEFAULT_FRAMEBUFFER	"/dev/fb0"
 
 
     /*
@@ -47,16 +48,16 @@ struct inode;
 
 static const char *ProgramName;
 
-static int Opt_now = 0;
+static int Opt_test = 0;
 static int Opt_show = 0;
-static int Opt_showonly = 1;
 static int Opt_info = 0;
 static int Opt_version = 0;
 static int Opt_verbose = 0;
 static int Opt_xfree86 = 0;
+static int Opt_change = 0;
+static int Opt_all = 0;
 
-static const char *Opt_ifb = NULL;
-static const char *Opt_ofb = NULL;
+static const char *Opt_fb = NULL;
 const char *Opt_modedb = DEFAULT_MODEDBFILE;
 static const char *Opt_xres = NULL;
 static const char *Opt_yres = NULL;
@@ -70,9 +71,11 @@ static const char *Opt_upper = NULL;
 static const char *Opt_lower = NULL;
 static const char *Opt_hslen = NULL;
 static const char *Opt_vslen = NULL;
+static const char *Opt_accel = NULL;
 static const char *Opt_hsync = NULL;
 static const char *Opt_vsync = NULL;
 static const char *Opt_csync = NULL;
+static const char *Opt_gsync = NULL;
 static const char *Opt_extsync = NULL;
 static const char *Opt_bcast = NULL;
 static const char *Opt_laced = NULL;
@@ -85,8 +88,7 @@ static struct {
     const char *name;
     const char **value;
 } Options[] = {
-    { "-ifb", &Opt_ifb },
-    { "-ofb", &Opt_ofb },
+    { "-fb", &Opt_fb },
     { "-db", &Opt_modedb },
     { "-xres", &Opt_xres },
     { "-yres", &Opt_yres },
@@ -100,9 +102,11 @@ static struct {
     { "-lower", &Opt_lower },
     { "-hslen", &Opt_hslen },
     { "-vslen", &Opt_vslen },
+    { "-accel", &Opt_accel },
     { "-hsync", &Opt_hsync },
     { "-vsync", &Opt_vsync },
     { "-csync", &Opt_csync },
+    { "-gsync", &Opt_gsync },
     { "-extsync", &Opt_extsync },
     { "-bcast", &Opt_bcast },
     { "-laced", &Opt_laced },
@@ -120,6 +124,39 @@ static struct {
 struct VideoMode *VideoModes = NULL;
 
 
+    /*
+     *  Hardware Accelerators
+     */
+
+static struct accelentry {
+    __u32 id;
+    const char *name;
+} Accelerators[] = {
+    { FB_ACCEL_NONE, "No" },
+    { FB_ACCEL_ATARIBLITT, "Atari Blitter" },
+    { FB_ACCEL_AMIGABLITT, "Amiga Blitter" },
+    { FB_ACCEL_S3_TRIO64, "S3 Trio64" },
+    { FB_ACCEL_NCR_77C32BLT, "NCR 77C32BLT" },
+    { FB_ACCEL_S3_VIRGE, "S3 ViRGE" },
+    { FB_ACCEL_ATI_MACH64GX, "ATI Mach64GX" },
+    { FB_ACCEL_DEC_TGA, "DEC 21030 TGA" },
+    { FB_ACCEL_ATI_MACH64CT, "ATI Mach64CT" },
+    { FB_ACCEL_ATI_MACH64VT, "ATI Mach64VT" },
+    { FB_ACCEL_ATI_MACH64GT, "ATI Mach64GT" },
+    { FB_ACCEL_SUN_CREATOR, "Sun Creator/Creator3D" },
+    { FB_ACCEL_SUN_CGSIX, "Sun cg6" },
+    { FB_ACCEL_SUN_LEO, "Sun leo/zx" },
+    { FB_ACCEL_IMS_TWINTURBO, "IMS Twin Turbo" },
+    { FB_ACCEL_3DLABS_PERMEDIA2, "3Dlabs Permedia 2" },
+    { FB_ACCEL_MATROX_MGA2064W, "Matrox MGA2064W (Millennium)" },
+    { FB_ACCEL_MATROX_MGA1064SG, "Matrox MGA1064SG (Mystique)" },
+    { FB_ACCEL_MATROX_MGA2164W, "Matrox MGA2164W (Millennium II)" },
+    { FB_ACCEL_MATROX_MGA2164W_AGP, "Matrox MGA2164W (Millennium II AGP)" },
+    { FB_ACCEL_MATROX_MGAG100, "Matrox G100 (Productiva G100)" },
+    { FB_ACCEL_MATROX_MGAG200, "Matrox G200 (Millennium, Mystique)" },
+};
+
+
     /*
      *  Current Video Mode
      */
@@ -140,7 +177,7 @@ static void ConvertFromVideoMode(const struct VideoMode *vmode,
 				 struct fb_var_screeninfo *var);
 static void ConvertToVideoMode(const struct fb_var_screeninfo *var,
 			       struct VideoMode *vmode);
-static u_long atoboolean(const char *var);
+static int atoboolean(const char *var);
 static void ReadModeDB(void);
 static struct VideoMode *FindVideoMode(const char *name);
 static void ModifyVideoMode(struct VideoMode *vmode);
@@ -180,7 +217,7 @@ int OpenFrameBuffer(const char *name)
 
     if ((fh = open(name, O_RDONLY)) == -1)
 	Die("open %s: %s\n", name, strerror(errno));
-    return(fh);
+    return fh;
 }
 
 
@@ -240,10 +277,13 @@ static void ConvertFromVideoMode(const struct VideoMode *vmode,
     var->xres_virtual = vmode->vxres;
     var->yres_virtual = vmode->vyres;
     var->bits_per_pixel = vmode->depth;
-    if (Opt_now)
-	var->activate = FB_ACTIVATE_NOW;
-    else
+    if (Opt_test)
 	var->activate = FB_ACTIVATE_TEST;
+    else
+	var->activate = FB_ACTIVATE_NOW;
+    if (Opt_all)
+	var->activate = FB_ACTIVATE_ALL;
+    var->accel_flags = vmode->accel_flags;
     var->pixclock = vmode->pixclock;
     var->left_margin = vmode->left;
     var->right_margin = vmode->right;
@@ -257,6 +297,8 @@ static void ConvertFromVideoMode(const struct VideoMode *vmode,
 	var->sync |= FB_SYNC_VERT_HIGH_ACT;
     if (vmode->csync == HIGH)
 	var->sync |= FB_SYNC_COMP_HIGH_ACT;
+    if (vmode->gsync == HIGH)
+	var->sync |= FB_SYNC_ON_GREEN;
     if (vmode->extsync == TRUE)
 	var->sync |= FB_SYNC_EXT;
     if (vmode->bcast == TRUE)
@@ -290,6 +332,7 @@ static void ConvertToVideoMode(const struct fb_var_screeninfo *var,
     vmode->hsync = var->sync & FB_SYNC_HOR_HIGH_ACT ? HIGH : LOW;
     vmode->vsync = var->sync & FB_SYNC_VERT_HIGH_ACT ? HIGH : LOW;
     vmode->csync = var->sync & FB_SYNC_COMP_HIGH_ACT ? HIGH : LOW;
+    vmode->gsync = var->sync & FB_SYNC_ON_GREEN ? TRUE : FALSE;
     vmode->extsync = var->sync & FB_SYNC_EXT ? TRUE : FALSE;
     vmode->bcast = var->sync & FB_SYNC_BROADCAST ? TRUE : FALSE;
     vmode->laced = FALSE;
@@ -306,20 +349,22 @@ static void ConvertToVideoMode(const struct fb_var_screeninfo *var,
 }
 
 
-static u_long atoboolean(const char *var)
+static int atoboolean(const char *var)
 {
-    u_long value = 0;
+    int value = 0;
 
     if (!strcasecmp(var, "false") || !strcasecmp(var, "low") ||
+	!strcasecmp(var, "no") || !strcasecmp(var, "off") ||
 	!strcmp(var, "0"))
 	value = 0;
     else if (!strcasecmp(var, "true") || !strcasecmp(var, "high") ||
+	     !strcasecmp(var, "yes") || !strcasecmp(var, "on") ||
 	     !strcmp(var, "1"))
 	value = 1;
     else
 	Die("Invalid value `%s'\n", var);
 
-    return(value);
+    return value;
 }
 
 
@@ -367,7 +412,7 @@ static struct VideoMode *FindVideoMode(const char *name)
 	if (!strcmp(name, vmode->name))
 	    break;
 
-    return(vmode);
+    return vmode;
 }
 
 
@@ -377,7 +422,7 @@ static struct VideoMode *FindVideoMode(const char *name)
 
 static void ModifyVideoMode(struct VideoMode *vmode)
 {
-    u_long hstep = 8, vstep = 2;
+    u_int hstep = 8, vstep = 2;
 
     if (Opt_xres)
 	vmode->xres = strtoul(Opt_xres, NULL, 0);
@@ -389,6 +434,8 @@ static void ModifyVideoMode(struct VideoMode *vmode)
 	vmode->vyres = strtoul(Opt_vyres, NULL, 0);
     if (Opt_depth)
 	vmode->depth = strtoul(Opt_depth, NULL, 0);
+    if (Opt_accel)
+	vmode->accel_flags = atoboolean(Opt_accel) ? FB_ACCELF_TEXT : 0;
     if (Opt_pixclock)
 	vmode->pixclock = strtoul(Opt_pixclock, NULL, 0);
     if (Opt_left)
@@ -409,6 +456,8 @@ static void ModifyVideoMode(struct VideoMode *vmode)
 	vmode->vsync = atoboolean(Opt_vsync);
     if (Opt_csync)
 	vmode->csync = atoboolean(Opt_csync);
+    if (Opt_gsync)
+	vmode->gsync = atoboolean(Opt_gsync);
     if (Opt_extsync)
 	vmode->extsync = atoboolean(Opt_extsync);
     if (Opt_bcast)
@@ -453,16 +502,17 @@ static void ModifyVideoMode(struct VideoMode *vmode)
 
 static void DisplayVModeInfo(struct VideoMode *vmode)
 {
-    u_long res, sstart, send, total;
+    u_int res, sstart, send, total;
 
     puts("");
     if (!Opt_xfree86) {
 	puts("mode \"name\"");
-	printf("    # H: %5.3f kHz, V: %5.3f Hz\n", vmode->hrate/1000,
-	       vmode->vrate);
-	printf("    geometry %ld %ld %ld %ld %ld\n", vmode->xres, vmode->yres,
+	if (vmode->pixclock)
+	    printf("    # D: %5.3f MHz, H: %5.3f kHz, V: %5.3f Hz\n",
+		   vmode->drate/1E6, vmode->hrate/1E3, vmode->vrate);
+	printf("    geometry %d %d %d %d %d\n", vmode->xres, vmode->yres,
 	       vmode->vxres, vmode->vyres, vmode->depth);
-	printf("    timings %ld %ld %ld %ld %ld %ld %ld\n", vmode->pixclock,
+	printf("    timings %d %d %d %d %d %d %d\n", vmode->pixclock,
 	       vmode->left, vmode->right, vmode->upper, vmode->lower,
 	       vmode->hslen, vmode->vslen);
 	if (vmode->hsync)
@@ -471,6 +521,8 @@ static void DisplayVModeInfo(struct VideoMode *vmode)
 	    puts("    vsync high");
 	if (vmode->csync)
 	    puts("    csync high");
+	if (vmode->gsync)
+	    puts("    gsync true");
 	if (vmode->extsync)
 	    puts("    extsync true");
 	if (vmode->bcast)
@@ -482,41 +534,42 @@ static void DisplayVModeInfo(struct VideoMode *vmode)
 	puts("endmode\n");
     } else {
 	puts("Mode \"name\"");
-	printf("    # H: %5.3f kHz, V: %5.3f Hz\n", vmode->hrate/1000,
-	       vmode->vrate);
-	printf("    DotClock %5.3f\n", 1E6/vmode->pixclock+0.001);
+	if (vmode->pixclock) {
+	    printf("    # D: %5.3f MHz, H: %5.3f kHz, V: %5.3f Hz\n",
+		   vmode->drate/1E6, vmode->hrate/1E3, vmode->vrate);
+	    printf("    DotClock %5.3f\n", vmode->drate/1E6+0.001);
+	} else
+	    puts("    DotClock Unknown");
 	res = vmode->xres;
 	sstart = res+vmode->right;
 	send = sstart+vmode->hslen;
 	total = send+vmode->left;
-	printf("    HTimings %ld %ld %ld %ld\n", res, sstart, send, total);
+	printf("    HTimings %d %d %d %d\n", res, sstart, send, total);
 	res = vmode->yres;
 	sstart = res+vmode->lower;
 	send = sstart+vmode->vslen;
 	total = send+vmode->upper;
-	printf("    VTimings %ld %ld %ld %ld\n", res, sstart, send, total);
+	printf("    VTimings %d %d %d %d\n", res, sstart, send, total);
 	printf("    Flags   ");
 	if (vmode->laced)
-	    printf(" Interlace");
+	    printf(" \"Interlace\"");
 	if (vmode->dblscan)
-	    printf(" DoubleScan");
+	    printf(" \"DoubleScan\"");
 	if (vmode->hsync)
-	    printf(" +HSync");
+	    printf(" \"+HSync\"");
 	else
-	    printf(" -HSync");
+	    printf(" \"-HSync\"");
 	if (vmode->vsync)
-	    printf(" +VSync");
+	    printf(" \"+VSync\"");
 	else
-	    printf(" -VSync");
+	    printf(" \"-VSync\"");
 	if (vmode->csync)
-	    puts(" +CSync");
-	else
-	    puts(" -CSync");
+	    printf(" \"Composite\"");
 	if (vmode->extsync)
 	    puts("    # Warning: XFree86 doesn't support extsync\n");
 	if (vmode->bcast)
-	    puts("    # Warning: XFree86 doesn't support bcast\n");
-	puts("EndMode\n");
+	    printf(" \"bcast\"");
+	puts("\nEndMode\n");
     }
 }
 
@@ -527,11 +580,13 @@ static void DisplayVModeInfo(struct VideoMode *vmode)
 
 static void DisplayFBInfo(struct fb_fix_screeninfo *fix)
 {
+    int i;
+
     puts("Frame buffer device information:");
-    printf("    Name      : `%s'\n", fix->id);
-    printf("    Address   : 0x%08lx\n", fix->smem_start);
-    printf("    Size      : %ld\n", fix->smem_len);
-    printf("    Type      : ");
+    printf("    Name        : `%s'\n", fix->id);
+    printf("    Address     : %p\n", fix->smem_start);
+    printf("    Size        : %d\n", fix->smem_len);
+    printf("    Type        : ");
     switch (fix->type) {
 	case FB_TYPE_PACKED_PIXELS:
 	    puts("PACKED PIXELS");
@@ -543,12 +598,28 @@ static void DisplayFBInfo(struct fb_fix_screeninfo *fix)
 	    printf("INTERLEAVED PLANES (%d bytes interleave)\n",
 		   fix->type_aux);
 	    break;
+	case FB_TYPE_TEXT:
+	    switch (fix->type_aux) {
+		case FB_AUX_TEXT_MDA:
+		    puts("Monochrome text");
+		    break;
+		case FB_AUX_TEXT_CGA:
+		    puts("CGA/EGA/VGA Color text");
+		    break;
+		case FB_AUX_TEXT_S3_MMIO:
+		    puts("S3 MMIO fasttext");
+		    break;
+		default:
+		    printf("%d (UNKNOWN) text\n", fix->type_aux);
+		    break;
+	    }
+	    break;
 	default:
 	    printf("%d (UNKNOWN)\n", fix->type);
-	    printf("    Type_aux  : %d\n", fix->type_aux);
+	    printf("    Type_aux    : %d\n", fix->type_aux);
 	    break;
     }
-    printf("    Visual    : ");
+    printf("    Visual      : ");
     switch (fix->visual) {
 	case FB_VISUAL_MONO01:
 	    puts("MONO01");
@@ -568,18 +639,26 @@ static void DisplayFBInfo(struct fb_fix_screeninfo *fix)
 	case FB_VISUAL_STATIC_PSEUDOCOLOR:
 	    puts("STATIC PSEUDOCOLOR");
 	    break;
-	case FB_VISUAL_STATIC_DIRECTCOLOR:
-	    puts("STATIC DIRECTCOLOR");
-	    break;
 	default:
 	    printf("%d (UNKNOWN)\n", fix->visual);
 	    break;
     }
-    printf("    XPanStep  : %d\n", fix->xpanstep);
-    printf("    YPanStep  : %d\n", fix->ypanstep);
-    printf("    YWrapStep : %d\n", fix->ywrapstep);
-    printf("    LineLength: %ld\n", fix->line_length);
-    puts("");
+    printf("    XPanStep    : %d\n", fix->xpanstep);
+    printf("    YPanStep    : %d\n", fix->ypanstep);
+    printf("    YWrapStep   : %d\n", fix->ywrapstep);
+    printf("    LineLength  : %d\n", fix->line_length);
+    if (fix->mmio_len) {
+	printf("    MMIO Address: %p\n", fix->mmio_start);
+	printf("    MMIO Size   : %d\n", fix->mmio_len);
+    }
+    printf("    Accelerator : ");
+    for (i = 0; i < sizeof(Accelerators)/sizeof(*Accelerators); i++)
+	if (fix->accel == Accelerators[i].id)
+	    break;
+    if (i < sizeof(Accelerators)/sizeof(*Accelerators))
+	puts(Accelerators[i].name);
+    else
+	printf("Unknown (%d)\n", fix->accel);
 }
 
 
@@ -589,21 +668,28 @@ static void DisplayFBInfo(struct fb_fix_screeninfo *fix)
 
 static int FillScanRates(struct VideoMode *vmode)
 {
-    u_long htotal = vmode->left+vmode->xres+vmode->right+vmode->hslen;
-    u_long vtotal = vmode->upper+vmode->yres+vmode->lower+vmode->vslen;
+    u_int htotal = vmode->left+vmode->xres+vmode->right+vmode->hslen;
+    u_int vtotal = vmode->upper+vmode->yres+vmode->lower+vmode->vslen;
 
     if (vmode->dblscan)
 	vtotal <<= 2;
     else if (!vmode->laced)
 	vtotal <<= 1;
 
-    if (!htotal || !vtotal || !vmode->pixclock)
-	return(0);
+    if (!htotal || !vtotal)
+	return 0;
 
-    vmode->hrate = 1E12/(htotal*vmode->pixclock);
-    vmode->vrate = vmode->hrate/vtotal*2;
+    if (vmode->pixclock) {
+	vmode->drate = 1E12/vmode->pixclock;
+	vmode->hrate = vmode->drate/htotal;
+	vmode->vrate = vmode->hrate/vtotal*2;
+    } else {
+	vmode->drate = 0;
+	vmode->hrate = 0;
+	vmode->vrate = 0;
+    }
 
-    return(1);
+    return 1;
 }
 
 
@@ -618,18 +704,17 @@ static void Usage(void)
 	"Valid options:\n"
 	"  General options:\n"
 	"    -h, --help         : display this usage information\n"
-	"    -n, --now          : change the video mode immediately\n"
-	"                         (default if -fb is not used)\n"
+	"    --test             : don't change, just test whether the mode is "
+				 "valid\n"
 	"    -s, --show         : display video mode settings\n"
 	"    -i, --info         : display all frame buffer information\n"
 	"    -v, --verbose      : verbose mode\n"
 	"    -V, --version      : print version information\n"
 	"    -x, --xfree86      : XFree86 compatibility mode\n"
+	"    -a, --all          : change all virtual consoles on this device\n"
 	"  Frame buffer special device nodes:\n"
-	"    -ofb <device>      : processed frame buffer device\n"
+	"    -fb <device>       : processed frame buffer device\n"
 	"                         (default is " DEFAULT_FRAMEBUFFER ")\n"
-	"    -ifb <device>      : input frame buffer device\n"
-	"                         (default is same as processed device)\n"
 	"  Video mode database:\n"
 	"    -db <file>         : video mode database file\n"
 	"                         (default is " DEFAULT_MODEDBFILE ")\n"
@@ -650,15 +735,19 @@ static void Usage(void)
 	"    -vslen <value>     : vertical sync length (in pixel lines)\n"
 	"    -t, --timings ...  : set all timing parameters at once\n"
 	"  Display flags:\n"
+	"    -accel <value>     : hardware text acceleration enable (false or "
+				 "true)\n"
 	"    -hsync <value>     : horizontal sync polarity (low or high)\n"
 	"    -vsync <value>     : vertical sync polarity (low or high)\n"
 	"    -csync <value>     : composite sync polarity (low or high)\n"
+	"    -gsync <value>     : synch on green (false or true)\n"
 	"    -extsync <value>   : external sync enable (false or true)\n"
 	"    -bcast <value>     : broadcast enable (false or true)\n"
 	"    -laced <value>     : interlace enable (false or true)\n"
 	"    -double <value>    : doublescan enable (false or true)\n"
 	"  Display positioning:\n"
-	"    -move <direction>  : move the visible part (left, right, up or down)\n"
+	"    -move <direction>  : move the visible part (left, right, up or "
+				 "down)\n"
 	"    -step <value>      : step increment (in pixels or pixel lines)\n"
 	"                         (default is 8 horizontal, 2 vertical)\n",
 	ProgramName);
@@ -686,21 +775,21 @@ int main(int argc, char *argv[])
 	argv++;
 	if (!strcmp(argv[0], "-h") || !strcmp(argv[0], "--help"))
 	    Usage();
-	else if (!strcmp(argv[0], "-n") || !strcmp(argv[0], "--now")) {
-	    Opt_now = 1;
-	    Opt_showonly = 0;
-	} else if (!strcmp(argv[0], "-s") || !strcmp(argv[0], "--show"))
+	else if (!strcmp(argv[0], "-v") || !strcmp(argv[0], "--verbose"))
+	    Opt_verbose = 1;
+	else if (!strcmp(argv[0], "-V") || !strcmp(argv[0], "--version"))
+	    Opt_version = 1;
+	else if (!strcmp(argv[0], "--test"))
+	    Opt_test = 1;
+	else if (!strcmp(argv[0], "-s") || !strcmp(argv[0], "--show"))
 	    Opt_show = 1;
 	else if (!strcmp(argv[0], "-i") || !strcmp(argv[0], "--info")) {
 	    Opt_show = 1;
 	    Opt_info = 1;
-	} else if (!strcmp(argv[0], "-v") || !strcmp(argv[0], "--verbose"))
-	    Opt_verbose = 1;
-	else if (!strcmp(argv[0], "-V") || !strcmp(argv[0], "--version")) {
-	    Opt_version = 1;
-	    Opt_showonly = 0;
 	} else if (!strcmp(argv[0], "-x") || !strcmp(argv[0], "--xfree86"))
 	    Opt_xfree86 = 1;
+	else if (!strcmp(argv[0], "-a") || !strcmp(argv[0], "--all"))
+	    Opt_all = 1;
 	else if (!strcmp(argv[0], "-g") || !strcmp(argv[0], "--geometry"))
 	    if (argc > 5) {
 		Opt_xres = argv[1];
@@ -708,9 +797,9 @@ int main(int argc, char *argv[])
 		Opt_vxres = argv[3];
 		Opt_vyres = argv[4];
 		Opt_depth = argv[5];
+		Opt_change = 1;
 		argc -= 5;
 		argv += 5;
-		Opt_showonly = 0;
 	    } else
 		Usage();
 	else if (!strcmp(argv[0], "-t") || !strcmp(argv[0], "--timings"))
@@ -722,26 +811,26 @@ int main(int argc, char *argv[])
 		Opt_lower = argv[5];
 		Opt_hslen = argv[6];
 		Opt_vslen = argv[7];
+		Opt_change = 1;
 		argc -= 7;
 		argv += 7;
-		Opt_showonly = 0;
 	    } else
 		Usage();
 	else {
-	    if (strcmp(&argv[0][2], "fb"))
-		Opt_showonly = 0;
 	    for (i = 0; Options[i].name; i++)
 		if (!strcmp(argv[0], Options[i].name))
 		    break;
 	    if (Options[i].name)
 		if (argc-- > 1) {
 		    *Options[i].value = argv[1];
+		    Opt_change = 1;
 		    argv++;
 		} else
 		    Usage();
-	    else if (!Opt_modename)
+	    else if (!Opt_modename) {
 		Opt_modename = argv[0];
-	    else
+		Opt_change = 1;
+	    } else
 		Usage();
 	}
     }
@@ -749,26 +838,27 @@ int main(int argc, char *argv[])
     if (Opt_version || Opt_verbose)
 	puts(VERSION);
 
-    if (!Opt_ofb) {
-	Opt_ofb = DEFAULT_FRAMEBUFFER;
-	Opt_now = 1;
-    }
+    if (!Opt_fb)
+	Opt_fb = DEFAULT_FRAMEBUFFER;
 
-    if (!Opt_ifb)
-	Opt_ifb = Opt_ofb;
+    /*
+     *  Open the Frame Buffer Device
+     */
+
+    fh = OpenFrameBuffer(Opt_fb);
 
     /*
      *  Get the Video Mode
      */
 
     if (Opt_modename) {
+
 	/*
 	 *  Read the Video Mode Database
 	 */
 
 	ReadModeDB();
 
-
 	if (!(vmode = FindVideoMode(Opt_modename)))
 	    Die("Unknown video mode `%s'\n", Opt_modename);      
 
@@ -776,14 +866,13 @@ int main(int argc, char *argv[])
 	if (Opt_verbose)
 	    printf("Using video mode `%s'\n", Opt_modename);
     } else {
-	fh = OpenFrameBuffer(Opt_ifb);
 	GetVarScreenInfo(fh, &var);
 	ConvertToVideoMode(&var, &Current);
 	if (Opt_verbose)
-	    printf("Using current video mode from `%s'\n", Opt_ifb);
+	    printf("Using current video mode from `%s'\n", Opt_fb);
     }
 
-    if (!Opt_showonly) {
+    if (Opt_change) {
 
 	/*
 	 *  Optionally Modify the Video Mode
@@ -795,21 +884,18 @@ int main(int argc, char *argv[])
 	 *  Set the Video Mode
 	 */
 
-	CloseFrameBuffer(fh);
-	fh = OpenFrameBuffer(Opt_ofb);
 	ConvertFromVideoMode(&Current, &var);
 	if (Opt_verbose)
-	    printf("Setting video mode to `%s'\n", Opt_ofb);
+	    printf("Setting video mode to `%s'\n", Opt_fb);
 	SetVarScreenInfo(fh, &var);
 	ConvertToVideoMode(&var, &Current);
-
     }
 
     /*
      *  Display some Video Mode Information
      */
 
-    if (Opt_show || Opt_showonly)
+    if (Opt_show || !Opt_change)
 	DisplayVModeInfo(&Current);
 
     if (Opt_info) {
